<?php

namespace Drupal\monarch_hook_plugin;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\Core\Extension\ModuleHandlerInterface;

/**
 * The hook plugin manager.
 */
class HookPluginManager extends DefaultPluginManager {

  /**
   * Constructs a HookPluginManager object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    parent::__construct(
      'Plugin/Hook',
      $namespaces,
      $module_handler,
      'Drupal\monarch_hook_plugin\HookInterface',
      'Drupal\monarch_hook_plugin\Annotation\Hook'
    );

    $this->setCacheBackend($cache_backend, 'hook_info_plugins');
  }

  /**
   * Transform hook name into function name.
   *
   * @param string $hook
   *   The hook name.
   *
   * @return string
   *   The function name.
   */
  public function getFunctionName(string $hook) {
    $first = TRUE;
    return implode('', array_map(function ($v) use (&$first) {
      $v = strtolower($v);

      if ($first) {
        $first = FALSE;
        return $v;
      }

      return ucfirst($v);
    }, explode('_', $hook)));
  }

  /**
   * Get hook plugins for the specified hook.
   *
   * @return array
   *   The plugins that match the specified hook.
   */
  public function getPlugins(string $func_name, $min_weight = -INF, $max_weight = INF) {
    $plugins = [];

    $definitions = $this->getDefinitions();

    usort($definitions, function ($a, $b) {
      return $a['weight'] - $b['weight'];
    });

    foreach ($definitions as $definition) {
      if ($definition['id'] ?? NULL) {
        if (
          $definition['weight'] >= $min_weight &&
          $definition['weight'] < $max_weight &&
          class_exists($definition['class']) &&
          ($plugin = $this->createInstance($definition['id'])) &&
          method_exists($plugin, $func_name)
        ) {
          $plugins[] = $this->createInstance($definition['id']);
        }
      }
    }

    return $plugins;
  }

}
